-- Broker shared data specification
-- File name: broker_data.ads
-- Author: Jaime Prez Crespo <japecre@gsyc.escet.urjc.es>
-- Last modification: Jun 15, 2005
--
-- Copyright (c) 2005, Jaime Prez Crespo
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without modification,
-- are permitted provided that the following conditions are met:
-- - Redistributions of source code must retain the above copyright notice, this list
--   of conditions and the following disclaimer.
-- - Redistributions in binary form must reproduce the above copyright notice, this list
--   of conditions and the following disclaimer in the documentation and/or other materials
--   provided with the distribution.
-- - Neither the name of the Rey Juan Carlos University nor the names of its contributors
--   may be used to endorse or promote products derived from this software without specific
--   prior written permission.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
-- OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
-- AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-- CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-- DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
-- IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

with mod_auth_basic;
with mod_auth;
with lower_layer_udp;
with ada.strings.unbounded;

package broker_data is

    multicast_main_addr: lower_layer_udp.end_point_type;
    round: integer := 1;
    broker_identification: ada.strings.unbounded.unbounded_string := 
                           ada.strings.unbounded.to_unbounded_string("Broker 0.1"); 

    -- initial login page
    html_login: string := "<!DOCTYPE HTML PUBLIC ""-//IETF//DTD HTML 2.0//EN"">" & ascii.lf &
                          "<html><head>" & ascii.lf &
                          "<title>Juego de la bolsa</title>" & ascii.lf &
                          "</head><body>" & ascii.lf &
                          "<h3>Bienvenido</h3>" & ascii.lf &
                          "<p>Introduzca su nombre de usuario:</p>" & ascii.lf &
                          "<form method=post action=""/login"">" & ascii.lf &
                          "<input type=""text"" name=""name"" size=""20"" value="""">" & ascii.lf &
                          "&nbsp;<input type=""submit"" value=""enviar"">" & ascii.lf &
                          "</form>" & ascii.lf &
                          "</body></html>";
    -- error page
    html_error: string := "<!DOCTYPE HTML PUBLIC ""-//IETF//DTD HTML 2.0//EN"">" & ascii.lf &
                          "<html><head>" & ascii.lf &
                          "<title>Error: p&aacute;gina incorrecta</title>" & ascii.lf &
                          "</head><body>" & ascii.lf &
                          "<h3>Error</h3>" & ascii.lf &
                          "<p>La p&aacute;gina solicitada no existe en este servidor. Por favor, " & ascii.lf &
                          "vaya a la <a href=""/"">p&aacute;gina de entrada</a> para comenzar.</p>" & ascii.lf &
                          "</body></html>";
    -- options page
    html_uopts: string := "<!DOCTYPE HTML PUBLIC ""-//IETF//DTD HTML 2.0//EN"">" & ascii.lf &
                          "<html><head>" & ascii.lf &
                          "<title>Juego de la bolsa - Men&uacute; de opciones</title>" & ascii.lf &
                          "</head><body>" & ascii.lf &
                          "<h3>Bienvenido, $USERNAME$</h3><hr>" & ascii.lf &
                          "Informaci&oacute;n general:" & ascii.lf &
                          "<ul><li>N&uacute;mero de acciones: $STOCKS$.</li>" & ascii.lf &
                          "<li>Saldo actual: $MONEY$.</li>" & ascii.lf &
                          "<li>Ronda actual del juego: $ROUND$.</li></ul><hr>" & ascii.lf &
                          "<form method=post action=""/manage"">" & ascii.lf &
                          "Ofertas:" & ascii.lf &
                          "<table border=""0""><tr><td>Seleccione el tipo de operaci&oacute;n:</td>" &
                          "<td><select name=""type""><option>compra<option>venta</select></td></tr>" & ascii.lf &
                          "<tr><td>Seleccione la cantidad de acciones:</td>" &
                          "<td><input type=""text"" name=""quantity"" size=""10"" value=""0""></td></tr>" & ascii.lf &
                          "<tr><td>Seleccione el precio de las acciones:</td>" &
                          "<td><input type=""text"" name=""value"" size=""10"" value=""0""></td></tr>" & ascii.lf &
                          "<tr><td><input type=""hidden"" name=""offer"" value=""$OFFERID$"">" & ascii.lf &
                          "<input type=""hidden"" name=""session"" value=""$SESSIONID$""></td>" & ascii.lf &
                          "<td><input type=""submit"" value=""Enviar""></td></tr></table></form><hr>" & ascii.lf &
                          "Ronda actual:<br>" & ascii.lf & "$OFFER$" & ascii.lf &
                          "</body></html>";

    -- user data related to sessions
    type user_data is record
        login: ada.strings.unbounded.unbounded_string;
        -- initial values
        offer: integer := 1;
        money: integer := 100;
        stock: integer := 100;
        vstock: integer := 100;
        registered: integer := 0;
    end record;

    function get_user_data(data: user_data) return mod_auth_basic.auth_basic_info;
    -- translates user_data into comprehensible auth information

    package auth is new mod_auth(user_data,get_user_data);
    -- package for authenticating users

end broker_data;
