-- Gong control implementation
-- File name: gong_control.ads
-- Author: Jaime Prez Crespo <japecre@gsyc.escet.urjc.es>
-- Last modification: June 2, 2005
--
-- Copyright (c) 2005, Jaime Prez Crespo
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without modification,
-- are permitted provided that the following conditions are met:
-- - Redistributions of source code must retain the above copyright notice, this list
--   of conditions and the following disclaimer.
-- - Redistributions in binary form must reproduce the above copyright notice, this list
--   of conditions and the following disclaimer in the documentation and/or other materials
--   provided with the distribution.
-- - Neither the name of the Rey Juan Carlos University nor the names of its contributors
--   may be used to endorse or promote products derived from this software without specific
--   prior written permission.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
-- OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
-- AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-- CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-- DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
-- IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

with ada.text_io;
with ada.numerics.discrete_random;
with ada.strings.unbounded;
with ada.exceptions;
with ada.characters.latin_1;
with ada.strings.fixed;
with ada.calendar;
with broker_messages;
with gong_data;
with handlers;
with http_common;
with utils;

package body gong_control is

    package udp renames lower_layer_udp;
    package unb renames ada.strings.unbounded;
    package utl renames utils;
    package hdl renames handlers;
    use ada.strings.fixed;
    use broker_messages;
    use gong_data;

    function build_response(r: response) return string is
        str: unb.unbounded_string;
    begin
        case r is
        when ROOT =>
            str := http_common.status_line(http_common.OK,http_common.HTTP11);
            http_common.header_line(str,http_common.SERVER,gong_identification);
        when ERROR =>
            str := http_common.status_line(http_common.OK,http_common.HTTP11);
            http_common.header_line(str,http_common.SERVER,gong_identification);
            http_common.header_line(str,http_common.CONTENTLEN,
                                    unb.to_unbounded_string(integer'image(html_err'length)));
            unb.append(str,ascii.cr & ascii.lf);
            unb.append(str,html_err);
        when others =>
            null;
        end case;
        return unb.to_string(str);
    end;

    function http(request: http_request.request) return string is
        use type ada.calendar.time;
        str,r: unb.unbounded_string;
        index: natural;
        t: duration;
    begin
        if unb.to_string(request.uri.path) = "/" then
            str := unb.to_unbounded_string(build_response(ROOT));
            -- build response body
            r := unb.to_unbounded_string(html_inf);
            -- put current round
            index := unb.index(r,"$ROUND$");
            unb.replace_slice(r,index,index - 1 +
                              unb.length(unb.to_unbounded_string("$ROUND$")),
                              trim(integer'image(round),ada.strings.both));
            -- put time left
            index := unb.index(r,"$TIME$");
            t := round_delay - (ada.calendar.clock - last_round);
            unb.replace_slice(r,index,index - 1 +
                              unb.length(unb.to_unbounded_string("$TIME$")),
                              trim(duration'image(t),ada.strings.both));

            http_common.header_line(str,http_common.CONTENTLEN,
                                    unb.to_unbounded_string(integer'image(
                                    unb.length(r))));
            unb.append(str,ascii.cr & ascii.lf);
            unb.append(str,r);

            return unb.to_string(str);
        else    -- any other page
            return build_response(ERROR);
        end if;

    exception
        when e: others =>
            utl.error("http: unexpected error: " & ada.exceptions.exception_message(e));
            return build_response(ERROR);
    end;

    procedure bind_multicast(addr: udp.end_point_type) is
    begin
        multicast_main_addr := addr;
        udp.bind(addr,hdl.main_handler'access);
    end;

    procedure clock is
        buf: aliased udp.buffer_type(1024);
        m: message_gong;
    begin
        loop
            delay round_delay;
            round := round + 1;
            m.round_id := round;
            message'class'output(buf'access,message'class(m));
            udp.send(multicast_main_addr,buf'access);
            last_round := ada.calendar.clock;
            utl.debug("gong: new round started" & ascii.lf);
        end loop;
    end;

end gong_control;