-- Multicast channel handling implementation
-- File name: handlers.adb
-- Author: Jaime Prez Crespo <japecre@gsyc.escet.urjc.es>
-- Last modification: June 2, 2005
--
-- Copyright (c) 2005, Jaime Prez Crespo
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without modification,
-- are permitted provided that the following conditions are met:
-- - Redistributions of source code must retain the above copyright notice, this list
--   of conditions and the following disclaimer.
-- - Redistributions in binary form must reproduce the above copyright notice, this list
--   of conditions and the following disclaimer in the documentation and/or other materials
--   provided with the distribution.
-- - Neither the name of the Rey Juan Carlos University nor the names of its contributors
--   may be used to endorse or promote products derived from this software without specific
--   prior written permission.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
-- OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
-- AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-- CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-- DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
-- IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

with broker_messages;
with ada.exceptions;
with utils;

package body handlers is

    package msg renames broker_messages;
    package utl renames utils;

    procedure main_handler(from: lower_layer_udp.end_point_type;
                           to: lower_layer_udp.end_point_type;
                           data: access lower_layer_udp.buffer_type) is
    begin
        declare
            -- read buffer (should have something in the message hierarchy).
            recv_message: msg.message'class := msg.message'class'input(data);
        begin
            if recv_message in msg.message'class then
                msg.process(msg.message'class(recv_message));
            else
                utl.error("handlers.main_handler: " &
                          "bad message received " &
                          "(not in message'class), tag: ");
            end if;
        end;
    exception
        when except: others =>
            utl.error("handlers.main_handler: " &
                      "unexpected exception raised, " &
                      ada.exceptions.exception_name (except));
    end main_handler;

end handlers;

