-- HTTP common utilities implementation
-- File name: http_common.adb
-- Author: Jaime Prez Crespo <japecre@gsyc.escet.urjc.es>
-- Last modification: May 26, 2005
--
-- Copyright (c) 2005, Jaime Prez Crespo
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without modification,
-- are permitted provided that the following conditions are met:
-- - Redistributions of source code must retain the above copyright notice, this list
--   of conditions and the following disclaimer.
-- - Redistributions in binary form must reproduce the above copyright notice, this list
--   of conditions and the following disclaimer in the documentation and/or other materials
--   provided with the distribution.
-- - Neither the name of the Rey Juan Carlos University nor the names of its contributors
--   may be used to endorse or promote products derived from this software without specific
--   prior written permission.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
-- OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
-- AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-- CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-- DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
-- IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package body http_common is

    use ada.strings.unbounded;

    function status_line(code: http_code;
                         proto: http_protocol) return unbounded_string is
        str: unbounded_string;
    begin
        if proto = HTTP10 then
            str := to_unbounded_string("HTTP/1.0 ");
        elsif proto = HTTP11 then
            str := to_unbounded_string("HTTP/1.1 " );
        else
            raise unknown_proto;
        end if;

        case code is
        when CONTINUE =>
            append(str,"100 Continue");
        when SWITCHPROTO =>
            append(str,"101 Switching Protocols");
        when OK =>
            append(str,"200 OK");
        when CREATED =>
            append(str,"201 Created");
        when ACCEPTED =>
            append(str,"202 Accepted");
        when NOAUTHINFO =>
            append(str,"203 Non-Authoritative Information");
        when NOCONTENT =>
            append(str,"204 No Content");
        when RSTCONTENT =>
            append(str,"205 Reset Content");
        when PARCONTENT =>
            append(str,"206 Partial Content");
        when MULCHOICES =>
            append(str,"300 Multiple Choices");
        when MOVEDPERM =>
            append(str,"301 Moved Permanently");
        when FOUND =>
            append(str,"302 Found");
        when MOVEDTEMP =>
            append(str,"302 Moved Temporarily");
        when SEEOTHER =>
            append(str,"303 See Other");
        when NOTMODIFIED =>
            append(str,"304 Not Modified");
        when USEPROXY =>
            append(str,"305 Use Proxy");
        when TEMPREDIR =>
            append(str,"307 Temporary Redirect");
        when BADREQUEST =>
            append(str,"400 Bad Request");
        when UNAUTHORIZED =>
            append(str,"401 Unauthorized");
        when PAYREQUIRED =>
            append(str,"402 Pay Required");
        when FORBIDDEN =>
            append(str,"403 Forbidden");
        when NOTFOUND =>
            append(str,"404 Not Found");
        when METHODNALLOW =>
            append(str,"405 Method Not Allowed");
        when NOTACCEPT =>
            append(str,"406 Not Acceptable");
        when PROXYAUTHREQ =>
            append(str,"407 Proxy Authentication Required");
        when REQTIMEOUT =>
            append(str,"408 Request Timeout");
        when CONFLICT =>
            append(str,"409 Conflict");
        when GONE =>
            append(str,"410 Gone");
        when LENGTHREQ =>
            append(str,"411 Length Required");
        when PRECONDFAIL =>
            append(str,"412 Precondition Failed");
        when REQENTLARGE =>
            append(str,"413 Request Entity Too Large");
        when REQURILARGE =>
            append(str,"414 Request-URI Too Long");
        when NMEDIATYPE =>
            append(str,"415 Unsupported Media Type");
        when BADRANGE =>
            append(str,"416 Requested Range Not Satisfiable");
        when EXPECTFAIL =>
            append(str,"417 Expectation Failed");
        when INTERROR =>
            append(str,"500 Internal Server Error");
        when NOTIMPLEM =>
            append(str,"501 Not Implemented");
        when BADGATEWAY =>
            append(str,"502 Bad Gateway");
        when SERVICEUNA =>
            append(str,"503 Service Unavailable");
        when GWTIMEOUT =>
            append(str,"504 Gateway Timeout");
        when BADHTTPVER =>
            append(str,"505 HTTP Version Not Supported");
        when others =>
            raise unknown_code;
        end case;
        append(str,ascii.cr & ascii.lf);

        return str;
    end;

    procedure header_line(str: in out unbounded_string;
                          name: http_header; value: unbounded_string) is
    begin
        case name is
        when CACHECONTROL =>
            append(str,"Cache-Control: ");
        when CONNECTION =>
            append(str,"Connection: ");
        when DATE =>
            append(str,"Date: ");
        when HPRAGMA =>
            append(str,"Pragma: ");
        when TRAILER =>
            append(str,"Trailer: ");
        when TRANSFERENC =>
            append(str,"Transfer-Encoding: ");
        when UPGRADE =>
            append(str,"Upgrade: ");
        when VIA =>
            append(str,"Via: ");
        when WARNING =>
            append(str,"Warning: ");
        when HACCEPT =>
            append(str,"Accept: ");
        when ACCEPTCHARSET =>
            append(str,"Accept-Charset: ");
        when ACCEPTENC =>
            append(str,"Accept-Encoding: ");
        when ACCEPTLANG =>
            append(str,"Accept-Language: ");
        when AUTHORIZATION =>
            append(str,"Authorization: ");
        when EXPECT =>
            append(str,"Expect: ");
        when FROM =>
            append(str,"From: ");
        when HOST =>
            append(str,"Host: ");
        when IFMATCH =>
            append(str,"If-Match: ");
        when IFMODIFSINCE =>
            append(str,"If-Modified-Since: ");
        when IFNONEMATCH =>
            append(str,"If-None-Match: ");
        when IFRANGE =>
            append(str,"If-Range: ");
        when IFUNMODSINCE =>
            append(str,"If-Unmodified-Since");
        when MAXFORWARDS =>
            append(str,"Max-Forwards: ");
        when PROXYAUTHOR =>
            append(str,"Proxy-Author: ");
        when HRANGE =>
            append(str,"Range: ");
        when REFERER =>
            append(str,"Referer: ");
        when TE =>
            append(str,"TE: ");
        when USERAGENT =>
            append(str,"User-Agent: ");
        when ALLOW =>
            append(str,"Allow: ");
        when CONTENTENC =>
            append(str,"Content-Encoding: ");
        when CONTENTLANG =>
            append(str,"Content-Language: ");
        when CONTENTLEN =>
            append(str,"Content-Length: ");
        when CONTENTLOC =>
            append(str,"Content-Location: ");
        when CONTENTMD5 =>
            append(str,"Content-MD5: ");
        when CONTENTRANGE =>
            append(str,"Content-Range: ");
        when CONTENTTYPE =>
            append(str,"Content-Type: ");
        when EXPIRES =>
            append(str,"Expires: ");
        when LASTMODIFIED =>
            append(str,"Last-Modified: ");
        when LOCATION =>
            append(str,"Location: ");
        when SERVER =>
            append(str,"Server: ");
        when WWWAUTH =>
            append(str,"WWW-Authenticate: ");
        when ACCEPTRANGES =>
            append(str,"Accept-Ranges: ");
        when AGE =>
            append(str,"Age: ");
        when ETAG =>
            append(str,"ETag: ");
        when PROXYAUTHEN =>
            append(str,"Proxy-Authenticate: ");
        when RETRYAFTER =>
            append(str,"Retry-After: ");
        when VARY =>
            append(str,"Vary: ");
        when PROXYCONN =>
            append(str,"Proxy-Connection: ");
        when KEEPALIVE =>
            append(str,"Keep-Alive: ");
        when others =>
            raise unknown_header;
        end case;
        append(str,value);
        append(str,ascii.cr & ascii.lf);
    end;

    procedure header_line(str: in out unbounded_string;
                          name: unbounded_string;
                          value: unbounded_string) is
    begin
        append(str,name);
        append(str,value);
        append(str,ascii.cr & ascii.lf);
    end;

    function get_post_var(name: string; str: unbounded_string) return string is
        beginning, next: natural;
        s: unbounded_string := str;
    begin
        beginning := index(s,name & "=");
        if beginning > 0 then
            delete(s,1,beginning + length(to_unbounded_string(name))); -- "*name="
            -- search the next var, if exists
            next := index(s,"&");
            if next > 0 then
                s := to_unbounded_string(slice(s,1,next - 1));
            end if;
        else
            raise unknown_var;
        end if;

        return to_string(s);
    end;

end http_common;
