-- HTTP common utilities specification
-- File name: http_common.ads
-- Author: Jaime Prez Crespo <japecre@gsyc.escet.urjc.es>
-- Last modification: May 25, 2005
--
-- Copyright (c) 2005, Jaime Prez Crespo
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without modification,
-- are permitted provided that the following conditions are met:
-- - Redistributions of source code must retain the above copyright notice, this list
--   of conditions and the following disclaimer.
-- - Redistributions in binary form must reproduce the above copyright notice, this list
--   of conditions and the following disclaimer in the documentation and/or other materials
--   provided with the distribution.
-- - Neither the name of the Rey Juan Carlos University nor the names of its contributors
--   may be used to endorse or promote products derived from this software without specific
--   prior written permission.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
-- OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
-- AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-- CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-- DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
-- IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

with ada.strings.unbounded;

package http_common is

    package unbounded renames ada.strings.unbounded;

    bad_syntax:     exception;
    unknown_method: exception;
    unknown_proto:  exception;
    unknown_code:   exception;
    unknown_header: exception;
    unknown_var:    exception;

    -- http protocol versions supported
    type http_protocol is (HTTP10, HTTP11);

    -- supported http/1.0 and http/1.1 method types
    type http_method is (OPTIONS, GET, HEAD, POST, PUT, DELETE, TRACE, CONNECT);

    -- known http/1.0 and http/1.1 status codes
    type http_code is (CONTINUE,        -- 100 successfull
                       SWITCHPROTO,     -- 101
                       OK,              -- 200
                       CREATED,         -- 201
                       ACCEPTED,        -- 202
                       NOAUTHINFO,      -- 203
                       NOCONTENT,       -- 204
                       RSTCONTENT,      -- 205
                       PARCONTENT,      -- 206
                       MULCHOICES,      -- 300 redirection
                       MOVEDPERM,       -- 301
                       FOUND,           -- 302
                       MOVEDTEMP,       -- 303
                       SEEOTHER,        -- 304
                       NOTMODIFIED,     -- 305
                       USEPROXY,        -- 306
                       TEMPREDIR,       -- 307
                       BADREQUEST,      -- 400 client error
                       UNAUTHORIZED,    -- 401
                       PAYREQUIRED,     -- 402
                       FORBIDDEN,       -- 403
                       NOTFOUND,        -- 404
                       METHODNALLOW,    -- 405
                       NOTACCEPT,       -- 406
                       PROXYAUTHREQ,    -- 407
                       REQTIMEOUT,      -- 408
                       CONFLICT,        -- 409
                       GONE,            -- 410
                       LENGTHREQ,       -- 411
                       PRECONDFAIL,     -- 412
                       REQENTLARGE,     -- 413
                       REQURILARGE,     -- 414
                       NMEDIATYPE,      -- 415
                       BADRANGE,        -- 416
                       EXPECTFAIL,      -- 417
                       INTERROR,        -- 500 server error)
                       NOTIMPLEM,       -- 501
                       BADGATEWAY,      -- 502
                       SERVICEUNA,      -- 503
                       GWTIMEOUT,       -- 504
                       BADHTTPVER);     -- 505

    -- known http/1.0 and http/1.1 header fields
    type http_header is (CACHECONTROL,CONNECTION,DATE,HPRAGMA,TRAILER,TRANSFERENC,
                         UPGRADE,VIA,WARNING,HACCEPT,ACCEPTCHARSET,ACCEPTENC,
                         ACCEPTLANG,AUTHORIZATION,EXPECT,FROM,HOST,IFMATCH,
                         IFMODIFSINCE,IFNONEMATCH,IFRANGE,IFUNMODSINCE,
                         MAXFORWARDS,PROXYAUTHOR,HRANGE,REFERER,TE,USERAGENT,
                         ALLOW,CONTENTENC,CONTENTLANG,CONTENTLEN,CONTENTLOC,
                         CONTENTMD5,CONTENTRANGE,CONTENTTYPE,EXPIRES,LASTMODIFIED,
                         LOCATION,SERVER,WWWAUTH,ACCEPTRANGES,AGE,ETAG,
                         PROXYAUTHEN,RETRYAFTER,VARY,PROXYCONN,KEEPALIVE);

    function status_line(code: http_code;
                         proto: http_protocol) return unbounded.unbounded_string;
    -- initialize a response with desired status code and protocol

    procedure header_line(str: in out unbounded.unbounded_string;
                          name: http_header; value: unbounded.unbounded_string);
    -- APPEND a header with a given value to a previously initialized string

    procedure header_line(str: in out unbounded.unbounded_string;
                          name: unbounded.unbounded_string;
                          value: unbounded.unbounded_string);
    -- APPEND a CUSTOM header to a previously initialized string

    function get_post_var(name: string; str: unbounded.unbounded_string) return string;
    -- returns the value of a post var specified

end http_common;
