-- HTTP_Server package implementation
-- File name: http_server.adb
-- Author: Jaime Prez Crespo <japecre@gsyc.escet.urjc.es>
-- Last modification: May 27, 2005
--
-- Copyright (c) 2005, Jaime Prez Crespo
-- All rights reserved.
--
-- Redistribution and use in source and binary forms, with or without modification,
-- are permitted provided that the following conditions are met:
-- - Redistributions of source code must retain the above copyright notice, this list
--   of conditions and the following disclaimer.
-- - Redistributions in binary form must reproduce the above copyright notice, this list
--   of conditions and the following disclaimer in the documentation and/or other materials
--   provided with the distribution.
-- - Neither the name of the Rey Juan Carlos University nor the names of its contributors
--   may be used to endorse or promote products derived from this software without specific
--   prior written permission.
--
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
-- OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
-- AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-- CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-- DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
-- IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

with ada.text_io;
with ada.characters.latin_1;
with ada.strings.unbounded;
with utils;

package body http_server is

    use http_request;
    package asu renames ada.strings.unbounded;
    package utl renames utils;

    function read(conn: tcon) return request is
        done, almost_done,cr: boolean := false;
        byte: character;
        buf: asu.unbounded_string;
        req: request;
        trim: boolean := true;
    begin
        while not done loop
            character'read (conn.descriptor'access, byte);
            if trim then
                case byte is
                when ada.characters.latin_1.cr =>
                    cr := true;
                when ada.characters.latin_1.lf =>
                    if cr then
                        trim := false;
                    end if;
                when others =>
                    trim := false;
                    ada.strings.unbounded.append(buf, byte);
                end case;
            else
                case byte is
                when ada.characters.latin_1.lf =>
                    if almost_done then
                        done := true;
                    else
                        almost_done := true;
                    end if;
                when ada.characters.latin_1.cr =>
                    null;
                when others =>
                    almost_done := false;
                end case;
                ada.strings.unbounded.append(buf, byte);
            end if;
        end loop;
        analyze_request(buf, req);
        conn.close := req.close;

        -- read body
        if req.datalen /= 0 then
            for i in 1..req.datalen loop
                character'read(conn.descriptor'access, byte);
                ada.strings.unbounded.append(req.data, byte);
            end loop;
        end if;

        return req;
    end;

    task body attend_task is
    begin
        loop    -- for persistent connections
            string'write(c.descriptor'access, process(read(c)));
            if c.close then
                tcp.dispose(c.descriptor);
                utl.debug("http_server: connection closed" & ada.characters.latin_1.lf);
                exit;
            end if;
            utl.debug("http_server: connection kept alive" & ada.characters.latin_1.lf);
        end loop;
    exception
        when e: others =>
            utl.debug("http_server: unexpected error: " &
                      ada.exceptions.exception_message(e) &
                      ada.characters.latin_1.lf);
            tcp.dispose(c.descriptor);
    end;

    task body ht is
        t: attend;
        c: tcon;
    begin
        tcp.listen_connection(addr);
        loop    -- main loop
            c := new tcon_record;
            tcp.wait_connection(addr, c.descriptor);
            utl.debug("http_server: connection accepted" & ada.characters.latin_1.lf);
            t := new attend_task(c);
        end loop;
    exception
        when e: others =>
            utl.debug("http_server: unexpected error: " &
                      ada.exceptions.exception_message(e) &
                      ada.characters.latin_1.lf);
            tcp.dispose(c.descriptor);
    end;

end http_server;

