-- HTTP_Server package specification
-- File name: http_server.ads
-- Author: Jaime Prez Crespo <japecre@gsyc.escet.urjc.es>
-- Last modification: May 27, 2005
--
-- Copyright (c) 2005, Jaime Prez Crespo
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without modification,
-- are permitted provided that the following conditions are met:
-- - Redistributions of source code must retain the above copyright notice, this list
--   of conditions and the following disclaimer.
-- - Redistributions in binary form must reproduce the above copyright notice, this list
--   of conditions and the following disclaimer in the documentation and/or other materials
--   provided with the distribution.
-- - Neither the name of the Rey Juan Carlos University nor the names of its contributors
--   may be used to endorse or promote products derived from this software without specific
--   prior written permission.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
-- OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
-- AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-- CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-- DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
-- IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

with lower_layer_tcp;
with ada.exceptions;
with ada.io_exceptions;
with http_request;

generic
    with function process(request: http_request.request) return string;
    -- this package must be instantiated by providing a function which takes
    -- a http request and generates an appropiate response, which MUST include
    -- status code and all headers needed. This server won't modify at all
    -- the response this function provides, so it will be sent over the network
    -- as is.

package http_server is
    package tcp renames lower_layer_tcp;

    --type ht is private;
    task type ht(addr: tcp.end_point);
    -- to run a task, declare a http_task and do a new to ht
    --type http_task is private;
    type http_task is access ht;
    -- main task for a _multithreaded_ http server

private

    -- this is a dirty hack! lower layer does not admit passing a connection
    -- type to a task as argument, so encapsulate it into a record and pass
    -- a pointer to it (assuring connections are independent between tasks).
    type tcon_record is record
        descriptor: aliased tcp.connection;
        close: boolean := true;
    end record;
    type tcon is access tcon_record;

    --task type ht(addr: tcp.end_point);
    --type http_task is access ht;

    task type attend_task(c: tcon);
    type attend is access attend_task;

    function read(conn: tcon) return http_request.request;

end http_server;

