-- Generic authentication package implementation
-- File name: mod_auth.adb
-- Author: Jaime Prez Crespo <japecre@gsyc.escet.urjc.es>
-- Last modification: June 15, 2005
--
-- Copyright (c) 2005, Jaime Prez Crespo
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without modification,
-- are permitted provided that the following conditions are met:
-- - Redistributions of source code must retain the above copyright notice, this list
--   of conditions and the following disclaimer.
-- - Redistributions in binary form must reproduce the above copyright notice, this list
--   of conditions and the following disclaimer in the documentation and/or other materials
--   provided with the distribution.
-- - Neither the name of the Rey Juan Carlos University nor the names of its contributors
--   may be used to endorse or promote products derived from this software without specific
--   prior written permission.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
-- OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
-- AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-- CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-- DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
-- IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

with ada.numerics.discrete_random;
with ada.exceptions;

package body mod_auth is

    package unb renames ada.strings.unbounded;
    package rand is new ada.numerics.discrete_random(integer);

    function new_session return session is
        s: session;
        g: rand.generator;
    begin
        if nsessions < MAXSESSIONS then
            rand.reset(g);
            s.id := rand.random(g) mod integer'last;
            sessions(nsessions + 1).id := s.id;
            nsessions := nsessions + 1;
        else
            raise enotmem;
        end if;
        return s;
    end;

    procedure associate(s: in out session; data: auth_info_addr) is
    begin
        for i in 1..nsessions loop
            if sessions(i).id = s.id then
                sessions(i).priv := data;
                s := sessions(i);   -- assure coherent changes
                return;             -- break loop
            end if;
        end loop;
    end;

    function authenticate(s: session) return boolean is
    begin
        return true;
    end;

    function validate(id: integer) return session is
    begin
        for i in 1..nsessions loop
            if sessions(i).id = id then
                -- found, return!
                return sessions(i);
            end if;
        end loop;
        raise badsession;
    end;

    function search(usr: unb.unbounded_string) return session is
        use type ada.strings.unbounded.unbounded_string;
        nfo: mod_auth_basic.auth_basic_info;
    begin
        for i in 1..nsessions loop
            nfo := get_auth_info(sessions(i).priv.all);
            if nfo.username = usr then
                -- found, return!
                return sessions(i);
            end if;
        end loop;
        raise badsession;
    end;

    function get_id(s: session) return integer is
    begin
        return s.id;
    end;

    function get_info(s: session) return auth_info_addr is
    begin
        return s.priv;
    end;

    procedure finnish(s: session) is
    begin
        if nsessions = 0 then
            raise badsession;
        end if;

        for i in 1..nsessions loop
            if sessions(i).id = s.id then
                -- session found, remove
                if nsessions /= i then
                    sessions(i) := sessions(nsessions);
                end if;
                nsessions := nsessions - 1;
            end if;
        end loop;
    end;

end mod_auth;
