-- Observer software implementation
-- File name: observer.adb
-- Author: Jaime Prez Crespo <japecre@gsyc.escet.urjc.es>
-- Last modification: June 13, 2005
--
-- Copyright (c) 2005, Jaime Prez Crespo
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without modification,
-- are permitted provided that the following conditions are met:
-- - Redistributions of source code must retain the above copyright notice, this list
--   of conditions and the following disclaimer.
-- - Redistributions in binary form must reproduce the above copyright notice, this list
--   of conditions and the following disclaimer in the documentation and/or other materials
--   provided with the distribution.
-- - Neither the name of the Rey Juan Carlos University nor the names of its contributors
--   may be used to endorse or promote products derived from this software without specific
--   prior written permission.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
-- OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
-- AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-- CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-- DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
-- IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

with lower_layer_tcp;
with lower_layer_udp;
with ada.command_line;
with ada.exceptions;
with ada.calendar;
with http_server;
with broker_messages;
with observer_control;
with observer_data;
with utils;

procedure observer is

    package tcp renames lower_layer_tcp;
    package udp renames lower_layer_udp;
    package cmd renames ada.command_line;
    package ctl renames observer_control;
    package utl renames utils;
    package web is new http_server(ctl.http);
    use broker_messages;
    use observer_data;
    
    http_srv_addr:  tcp.end_point;
    http_srv_task:  web.http_task;
    multicast_addr: udp.end_point_type;

begin
    if cmd.argument_count /= 3 then
        utl.error("observer: wrong parameter list" & ascii.lf);
        utl.error("Use: observer MULTICAST_IP MULTICAST_PORT HTTP_PORT");
    else
        -- initialize params
        current_entity := OBSERVER;

        -- build multicast end point
        multicast_addr := udp.build(cmd.argument(1),
                                    positive'value(cmd.Argument(2)));
        ctl.bind_multicast(multicast_addr);

        -- build http end point and run server on it
        http_srv_addr := tcp.build(udp.to_ip(udp.get_host_name),
                                   positive'value(cmd.argument(3)));
        utl.set_debug_mode;
        http_srv_task := new web.ht(http_srv_addr);
    end if;
exception
    when e: others =>
        utl.error("observer: unexpected exception found: " & ada.exceptions.exception_message(e));
end observer;
