-- Observer shared data specification
-- File name: observer_data.ads
-- Author: Jaime Prez Crespo <japecre@gsyc.escet.urjc.es>
-- Last modification: June 16, 2005
--
-- Copyright (c) 2005, Jaime Prez Crespo
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without modification,
-- are permitted provided that the following conditions are met:
-- - Redistributions of source code must retain the above copyright notice, this list
--   of conditions and the following disclaimer.
-- - Redistributions in binary form must reproduce the above copyright notice, this list
--   of conditions and the following disclaimer in the documentation and/or other materials
--   provided with the distribution.
-- - Neither the name of the Rey Juan Carlos University nor the names of its contributors
--   may be used to endorse or promote products derived from this software without specific
--   prior written permission.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
-- OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
-- AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-- CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-- DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
-- IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

with lower_layer_udp;
with lists_generic;
with ada.strings.unbounded;

package observer_data is

    package unb renames ada.strings.unbounded;

    multicast_main_addr: lower_layer_udp.end_point_type;
    --round: integer := 1;
    observer_identification: ada.strings.unbounded.unbounded_string := 
                             ada.strings.unbounded.to_unbounded_string("Observer 0.1"); 

    -- initial login page
    html_inf: string := "<!DOCTYPE HTML PUBLIC ""-//IETF//DTD HTML 2.0//EN"">" & ascii.lf &
                        "<html><head>" & ascii.lf &
                        "<title>Observer</title>" & ascii.lf &
                        "</head><body>" & ascii.lf &
                        "<h3>Brokers registrados</h3>" & ascii.lf &
                        "<ul>$BROKERS$</ul>" & ascii.lf &
                        "<hr><h3>Operaciones</h3>" & ascii.lf &
                        "<ul>$OPERS$</ul>" & ascii.lf &
                        "</body></html>";
    -- error page
    html_err: string := "<!DOCTYPE HTML PUBLIC ""-//IETF//DTD HTML 2.0//EN"">" & ascii.lf &
                        "<html><head>" & ascii.lf &
                        "<title>Error: p&aacute;gina incorrecta</title>" & ascii.lf &
                        "</head><body>" & ascii.lf &
                        "<h3>Error</h3>" & ascii.lf &
                        "<p>La p&aacute;gina solicitada no existe en este servidor. Por favor, " & ascii.lf &
                        "vaya a la <a href=""/"">p&aacute;gina raiz</a>.</p>" & ascii.lf &
                        "</body></html>";

    ebadbroker: exception;

    type broker_queue is record
        id: unb.unbounded_string;
        money:  integer := 100;
        stock:  integer := 100;
        vmoney: integer := 100;
        vstock: integer := 100;
    end record;
    function broker_image(q: broker_queue) return string;
    package bl is new lists_generic(broker_queue,broker_image);

    brokers: bl.list := bl.null_list;

    function search(l: bl.list; id: unb.unbounded_string) return bl.reference;
    -- search a broker on a given list
    -- may raise ebadbroker

end observer_data;
