-- General utilities implementation
-- File name: utils.adb
-- Author: Jaime Prez Crespo <japecre@gsyc.escet.urjc.es>
-- Last modification: May 30, 2005
--
-- Copyright (c) 2005, Jaime Prez Crespo
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without modification,
-- are permitted provided that the following conditions are met:
-- - Redistributions of source code must retain the above copyright notice, this list
--   of conditions and the following disclaimer.
-- - Redistributions in binary form must reproduce the above copyright notice, this list
--   of conditions and the following disclaimer in the documentation and/or other materials
--   provided with the distribution.
-- - Neither the name of the Rey Juan Carlos University nor the names of its contributors
--   may be used to endorse or promote products derived from this software without specific
--   prior written permission.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
-- OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
-- AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-- CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-- DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
-- IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

with ada.text_io;
with ada.strings.fixed;

package body utils is

    use ada.strings.fixed;

    procedure set_debug_mode is
    begin
        dbg := true;
    end;

    procedure error(str: string) is
    begin
        ada.text_io.put_line(ada.text_io.current_error,str);
    end;

    procedure debug(str: string) is
    begin
        if dbg then
            -- print the message to current error output descriptor
            ada.text_io.put(ada.text_io.current_error, str);
        end if;
    end;

    function image(t: ada.calendar.time) return string is
        s: ada.calendar.day_duration;
    begin
        s := ada.calendar.seconds(t);
        return trim(integer'image(integer(s)/3600),ada.strings.both)
               & ":" &
               trim(integer'image((integer(s) mod 3600)/60),ada.strings.both)
               & ":" &
               trim(integer'image((integer(s) mod 3600) mod 60),ada.strings.both);
    end;

end utils;